class CollarColumn:
    HOLEID = 'holeid'
    PROJECTID = 'projectid'
    X = 'x'
    Y = 'y'
    Z = 'z'
    TDEPTH = 'tdepth'
    PROSPECT = 'prospect'

class SurveyColumn:
    HOLEID = 'holeid'
    PROJECTID = 'projectid'
    DEPTH = 'depth'
    AZIMUTH = 'azimuth'
    DIP = 'dip'

class AlterationColumn:
    HOLEID = 'holeid'
    PROJECTID = 'projectid'
    FROM = 'from'
    TO = 'to'

class AssayColumn:
    HOLEID = 'holeid'
    PROJECTID = 'projectid'
    SAMPLEID = 'sampleid'
    FROM = 'from'
    TO = 'to'

class LithoColumn:
    HOLEID = 'holeid'
    PROJECTID = 'projectid'
    FROM = 'from'
    TO = 'to'

class MineralisationColumn:
    HOLEID = 'holeid'
    PROJECTID = 'projectid'
    FROM = 'from'
    TO = 'to'


from geox.connection import connect_to_endpoint
from geox.entity.api_endpoint import APIEndpoint
from geox.http_response.http_project import HttpReponseProject


def get_project(api_key: str, project_id: str) -> HttpReponseProject:
    response = connect_to_endpoint(
        url=APIEndpoint.GET_PROJECT,
        method='GET',
        headers={
            "api-key": api_key,
        },
        params={
            'project_id': project_id
        }
    )
    
    return HttpReponseProject(response)
    
    


from geox.factory.factory import Factory
from geox.project_version import ProjectVersion
from geox.utils import convert_iso_8601_to_datetime


class ProjectVersionFactory(Factory):
    def __init__(self):
        ...
    
    
    def create(self, data):
        project_version = None
        hash = data['project_version_hash'] if 'project_version_hash' in data else None
        if hash:
            project_version = ProjectVersion(hash)
                
            project_version.id = data['project_version_id'] \
                if 'project_version_id' in data else None

            project_version.created_at = convert_iso_8601_to_datetime(data['created_at']) \
                if 'created_at' in data else None

            project_version.num_of_collar_rows = data['num_of_collar_rows'] \
                if 'num_of_collar_rows' in data else None

            project_version.num_of_survey_rows = data['num_of_survey_rows'] \
                if 'num_of_survey_rows' in data else None

            project_version.num_of_alteration_rows = data['num_of_alteration_rows'] \
                if 'num_of_alteration_rows' in data else None

            project_version.num_of_assay_rows = data['num_of_assay_rows'] \
                if 'num_of_assay_rows' in data else None

            project_version.num_of_litho_rows = data['num_of_litho_rows'] \
                if 'num_of_litho_rows' in data else None

            project_version.num_of_mineralisation_rows = data['num_of_mineralisation_rows'] \
                if 'num_of_mineralisation_rows' in data else None
                
        return project_version


from geox.connection import connect_to_endpoint
from geox.entity.api_endpoint import APIEndpoint
from geox.http_response.http_project_version import HttpReponseProjectVersion


def get_project_version(api_key: str, hash: str) -> HttpReponseProjectVersion:
    response = connect_to_endpoint(
        url=APIEndpoint.GET_PROJECT_VERSION,
        method='GET',
        headers={
            "api-key": api_key,
        },
        params={
            'hash': hash
        }
    )
    
    return HttpReponseProjectVersion(response)
    
    


from datetime import datetime
from geox.api_caller.get_project_version import get_project_version
from geox.api_caller.get_project_versions import get_project_versions
from geox.exceptions import APIKeyException, ProjectIDException
from geox.project_version import ProjectVersion
from typing import List


class Project:
    """Project, defined by project id, may contain various project versions
    """
    def __init__(self, project_id: str):
        self._check_project_id(project_id)
        
        self.project_id: str = project_id
        
        self.title: str = None
        self.location: str = None
        self.description: str = None
        self.created_at: datetime = None
        self.updated_at: datetime = None
        self.num_of_project_versions: int = None
        
        self.project_versions: List[ProjectVersion] = None
        
        # private variables
        self._api_key = None
        self._timestamp = datetime.now()
    
    
    def _check_project_id(self, project_id: str) -> None:
        if not project_id: raise ProjectIDException('Project ID is None or invalid')
        if not isinstance(project_id, str): raise ProjectIDException('Project ID should be a string')
    
    
    def _check_api_key(self, api_key):
        if not api_key: raise APIKeyException('API Key cannot be None')
        if not isinstance(api_key, str): raise APIKeyException('API Key should be string')
    
    
    def set_api_key(self, api_key: str) -> None:
        self._api_key = api_key
        
        
    def read_all_project_versions(self) -> List[ProjectVersion]:
        """Read all project versions of this project

        Returns:
            List[Project]: List of Project Versions
        """
        self._check_api_key(self._api_key)
        http_response = get_project_versions(self._api_key, self.project_id)
        self.project_versions = http_response.project_versions
        return self.project_versions


    def read_project_version(self, hash: str) -> ProjectVersion:
        """Read specific project version

        Args:
            hash (str): project version hash

        Returns:
            ProjectVersion: Project Version object
        """
        self._check_api_key(self._api_key)
        http_response = get_project_version(self._api_key, hash)
        if http_response.project_version: http_response.project_version.set_api_key(self._api_key)
        return http_response.project_version



from geox.exceptions import ServerErrorException


class HttpReponseDataset():
    def __init__(self, json_response: dict, dataset_type: str):
        self.data = json_response['data'] if 'data' in json_response else []
        self.max_result = json_response['max_result'] if 'max_result' in json_response else None
        self.dataset_type = dataset_type
        
    def _is_max_result_reached(self):
        if self.max_result is None:
            raise ServerErrorException('Cannot retrieve "max result" data from server.')
        
        return len(self.data) < self.max_result
    
    def get_next_index(self):
        next_index = -2
        if len(self.data) > 0 and not self._is_max_result_reached():
            next_index = self.data[-1]['id'] + 1

        return next_index


from geox.exceptions import APIKeyException, ParameterException, ServerErrorException
from http import HTTPStatus
from requests import request, Response, packages
from requests.exceptions import ConnectionError, SSLError
from time import sleep
from urllib3.exceptions import InsecureRequestWarning
import urllib3

urllib3.disable_warnings(InsecureRequestWarning)


def check_status_code(response: Response) -> None:
    '''checking all available status code from GeoX API'''
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        raise APIKeyException(f'Unauthorized. {response.json()["detail"]}.')
    
    elif response.status_code == HTTPStatus.BAD_REQUEST:
        raise ParameterException(f'Some of the parameter in request is invalid. {response.text}.')
    
    elif response.status_code == HTTPStatus.NOT_FOUND:
        raise ServerErrorException('404. Not Found response, server error.')


def connect_to_endpoint(url: str, method: str, headers: dict={}, params:dict ={}, verify_ssl:bool=True) -> dict:
    '''Connect to endpoint'''
    try:
        response = request(
            method=method, 
            url=url, 
            headers = headers, 
            params = params,
            verify=verify_ssl,
            )
        
        # checking status code
        check_status_code(response)

        return response.json()
    
    except ParameterException as e:
        print(e)
        # sys.exit()
        return {}
    
    except SSLError:
        return connect_to_endpoint(url, method, headers, params, verify_ssl=False)
        
    except ConnectionError:
        print('Connection Error. Retrying in 5 seconds')
        sleep(5)
        return connect_to_endpoint(url, method, headers, params)
